@@ -1,8 +1,5 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/mediafilespreviewer.py
-#
 # Copyright (c) 2011, Pier Carteri <pier.carteri@gmail.com>
 # Copyright (c) 2012, Thibault Saunier <tsaunier@gnome.org>
 #
@@ -20,23 +17,26 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-
+import html
 from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import GdkPixbuf
+from gi.repository import GES
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gst
 from gi.repository import Gtk
-from gi.repository import Gdk
-from gi.repository import GdkPixbuf
 from gi.repository import Pango
-from gi.repository import GstPbutils
 
 from pitivi.settings import GlobalSettings
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import uri_is_valid
 from pitivi.utils.pipeline import AssetPipeline
-from pitivi.utils.ui import beautify_length, beautify_stream, SPACING
-from pitivi.viewer import ViewerWidget
+from pitivi.utils.ui import beautify_length
+from pitivi.utils.ui import beautify_stream
+from pitivi.utils.ui import SPACING
+from pitivi.viewer.viewer import ViewerWidget
 
 PREVIEW_WIDTH = 250
 PREVIEW_HEIGHT = 100
@@ -55,7 +55,7 @@ GlobalSettings.addConfigOption('FCpreviewHeight',
                                key='video-preview-height',
                                default=PREVIEW_HEIGHT)
 
-acceptable_tags = [
+ACCEPTABLE_TAGS = [
     Gst.TAG_ALBUM_ARTIST,
     Gst.TAG_ARTIST,
     Gst.TAG_TITLE,
@@ -69,12 +69,10 @@ acceptable_tags = [
 
 
 class PreviewWidget(Gtk.Grid, Loggable):
+    """Widget for displaying a GStreamer sink with playback controls.
 
-    """
-    Widget for displaying a GStreamer sink with playback controls.
-
-    @ivar settings: The settings of the app.
-    @type settings: L{GlobalSettings}
+    Args:
+        settings (GlobalSettings): The settings of the app.
     """
 
     def __init__(self, settings, minimal=False):
@@ -85,10 +83,7 @@ class PreviewWidget(Gtk.Grid, Loggable):
         self.connect('destroy', self._destroy_cb)
 
         self.settings = settings
-        self.preview_cache = {}
-        self.preview_cache_errors = {}
-
-        self.discoverer = GstPbutils.Discoverer.new(Gst.SECOND)
+        self.error_message = None
 
         # playbin for play pics
         self.player = AssetPipeline(clip=None, name="preview-player")
@@ -98,17 +93,18 @@ class PreviewWidget(Gtk.Grid, Loggable):
 
         # some global variables for preview handling
         self.is_playing = False
+        self.at_eos = False
         self.original_dims = (PREVIEW_WIDTH, PREVIEW_HEIGHT)
         self.countinuous_seek = False
         self.slider_being_used = False
         self.current_selected_uri = ""
         self.current_preview_type = ""
+        self.play_on_discover = False
         self.description = ""
-        self.tags = {}
 
         # Gui elements:
         # Drawing area for video output
-        self.preview_video = ViewerWidget(self.player.video_sink)
+        self.preview_video = ViewerWidget(self.player.sink_widget)
         self.preview_video.props.hexpand = minimal
         self.preview_video.props.vexpand = minimal
         self.preview_video.show_all()
@@ -175,9 +171,13 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.bbox.remove(self.b_zoom_in)
             self.bbox.remove(self.b_zoom_out)
 
-    def add_preview_request(self, dialogbox):
-        """add a preview request """
-        uri = dialogbox.get_preview_uri()
+    def update_preview_cb(self, file_chooser):
+        """Previews the URI of the specified file chooser.
+
+        Args:
+            file_chooser (Gtk.FileChooser): The file chooser providing the URI.
+        """
+        uri = file_chooser.get_preview_uri()
         if uri is None or not uri_is_valid(uri):
             return
         self.previewUri(uri)
@@ -186,37 +186,27 @@ class PreviewWidget(Gtk.Grid, Loggable):
         self.log("Preview request for %s", uri)
         self.clear_preview()
         self.current_selected_uri = uri
-        if uri in self.preview_cache:  # Already discovered
-            self.log(uri + " already in cache")
-            self.show_preview(uri, None)
-        elif uri in self.preview_cache_errors:
-            self.log(uri + " already in error cache")
-            self.show_error(uri)
-        else:
-            self.log("Call discoverer for " + uri)
-            self.fixme("Use a GESAsset here, and discover async with it")
-            try:
-                info = self.discoverer.discover_uri(uri)
-            except Exception as e:
-                self.preview_cache_errors[uri] = e
-                if self.current_selected_uri == uri:
-                    self.show_error(uri)
-                return
+        GES.UriClipAsset.new(uri, None, self.__asset_loaded_cb)
 
-            if self.current_selected_uri == uri:
-                self.show_preview(uri, info)
-
-    def show_preview(self, uri, info):
-        if info:
-            self.preview_cache[uri] = info
-        else:
-            self.log("Show preview for " + uri)
-            info = self.preview_cache.get(uri, None)
-
-        if info is None:
-            self.log("No preview for " + uri)
+    def __asset_loaded_cb(self, source, res):
+        uri = source.get_id()
+        try:
+            asset = GES.Asset.request_finish(res)
+        except GLib.Error as error:
+            self.log("Failed discovering %s: %s", uri, error.message)
+            self.error_message = error.message
+            self._show_error(uri)
             return
 
+        self.log("Discovered %s", uri)
+        self.error_message = None
+        self._show_preview(uri, asset.get_info())
+        if self.play_on_discover:
+            self.play_on_discover = False
+            self.play()
+
+    def _show_preview(self, uri, info):
+        self.log("Show preview for %s", uri)
         duration = info.get_duration()
         pretty_duration = beautify_length(duration)
 
@@ -253,8 +243,8 @@ class PreviewWidget(Gtk.Grid, Loggable):
                 video_height = video.get_height()
                 w, h = self.__get_best_size(video_width, video_height)
                 self.preview_video.set_size_request(w, h)
-                self.preview_video.setDisplayAspectRatio(
-                    float(video_width) / video_height)
+                aspect_ratio = video_width / video_height
+                self.preview_video.setDisplayAspectRatio(aspect_ratio)
                 self.preview_video.show()
                 self.bbox.show()
                 self.play_button.show()
@@ -269,7 +259,6 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.current_preview_type = 'audio'
             self.preview_video.hide()
             audio = info.get_audio_streams()
-
             if not audio:
                 return
 
@@ -291,11 +280,18 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.b_zoom_out.hide()
             self.bbox.show()
 
-    def show_error(self, unused_uri):
+    def _show_error(self, unused_uri):
         self.l_error.show()
         self.b_details.show()
 
     def play(self):
+        if not self.current_preview_type:
+            self.play_on_discover = True
+            return
+        if self.at_eos:
+            # The content played once already and the pipeline is at the end.
+            self.at_eos = False
+            self.player.simple_seek(0)
         self.player.setState(Gst.State.PLAYING)
         self.is_playing = True
         self.play_button.set_stock_id(Gtk.STOCK_MEDIA_PAUSE)
@@ -324,7 +320,6 @@ class PreviewWidget(Gtk.Grid, Loggable):
         self.description = ""
         self.l_tags.set_markup("")
         self.pause(state=Gst.State.NULL)
-        self.tags = {}
         self.current_selected_uri = ""
         self.current_preview_type = ""
         self.preview_image.hide()
@@ -340,6 +335,7 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.countinuous_seek = False
             value = int(widget.get_value())
             self.player.simple_seek(value)
+            self.at_eos = False
             if self.is_playing:
                 self.player.setState(Gst.State.PLAYING)
             # Now, allow gobject timeout to continue updating the slider pos:
@@ -349,10 +345,14 @@ class PreviewWidget(Gtk.Grid, Loggable):
         if self.countinuous_seek:
             value = int(widget.get_value())
             self.player.simple_seek(value)
+            self.at_eos = False
 
     def _pipelineEosCb(self, unused_pipeline):
+        self._update_position()
         self.pause()
-        self.pos_adj.set_value(0)
+        # The pipeline is at the end. Leave it like that so the last frame
+        # is displayed.
+        self.at_eos = True
 
     def _pipelineErrorCb(self, unused_pipeline, unused_message, unused_detail):
         self.pause(state=Gst.State.NULL)
@@ -405,8 +405,8 @@ class PreviewWidget(Gtk.Grid, Loggable):
             self.settings.FCpreviewWidth = int(w)
             self.settings.FCpreviewHeight = int(h)
 
-    def _appendTag(self, taglist, tag, unused_udata):
-        if tag in acceptable_tags:
+    def _append_tag(self, taglist, tag, tags):
+        if tag in ACCEPTABLE_TAGS:
             name = Gst.tag_get_nick(tag)
             type = Gst.tag_get_type(tag)
             type_getters = {GObject.TYPE_STRING: 'get_string',
@@ -415,36 +415,38 @@ class PreviewWidget(Gtk.Grid, Loggable):
                             GObject.TYPE_INT: 'get_int',
                             GObject.TYPE_UINT: 'get_uint'}
             if type in type_getters:
-                if type == GObject.TYPE_STRING:
-                    value = getattr(taglist, type_getters[type])(tag)[1]
-                    value = value.replace('<', ' ').replace('>', ' ')
-                else:
-                    value = str(getattr(taglist, type_getters[type])(tag)[1])
-                self.tags[name] = value
+                res, value = getattr(taglist, type_getters[type])(tag)
+                assert res
+                if not type == GObject.TYPE_STRING:
+                    value = str(value)
+                tags[name] = value
 
-    def _tag_found_cb(self, abus, mess):
-        tag_list = mess.parse_tag()
-        tag_list.foreach(self._appendTag, None)
-        keys = list(self.tags.keys())
-        keys.sort()
+    def _tag_found_cb(self, unused_bus, message):
+        tag_list = message.parse_tag()
+        tags = {}
+        tag_list.foreach(self._append_tag, tags)
+        items = list(tags.items())
+        items.sort()
         text = self.description + "\n\n"
-        for key in keys:
-            text = text + "<b>" + \
-                key.capitalize() + "</b>: " + self.tags[key] + "\n"
+        for key, value in items:
+            capitalized = key.capitalize()
+            escaped = html.escape(value)
+            text = text + "<b>%s</b>: %s\n" % (key, escaped)
         self.l_tags.set_markup(text)
 
     def _on_b_details_clicked_cb(self, unused_button):
-        mess = self.preview_cache_errors.get(self.current_selected_uri, None)
-        if mess is not None:
-            dialog = Gtk.MessageDialog(transient_for=None,
-                                       modal=True,
-                                       message_type=Gtk.MessageType.WARNING,
-                                       buttons=Gtk.ButtonsType.OK,
-                                       text=str(mess))
-            dialog.set_icon_name("pitivi")
-            dialog.set_title(_("Error while analyzing a file"))
-            dialog.run()
-            dialog.destroy()
+        if not self.error_message:
+            return
+
+        dialog = Gtk.MessageDialog(transient_for=None,
+                                   modal=True,
+                                   message_type=Gtk.MessageType.WARNING,
+                                   buttons=Gtk.ButtonsType.OK,
+                                   text=self.error_message)
+        dialog.set_icon_name("pitivi")
+        dialog.set_title(_("Error while analyzing a file"))
+        dialog.run()
+        dialog.destroy()
 
     def _destroy_cb(self, widget):
         self.player.release()
